﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Xml.Linq;

using Sycamores.XFrameworkCF;

namespace Sycamores.MobileShare.Client
{
    /// <summary>Application Data Element.</summary>
    public class ApplicationDataElement : ComplexType
    {
        private const string cTag = "applicationData";
        private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";

        /// <summary>Default constructor.</summary>
        /// <param name="element">Element object.</param>
        public ApplicationDataElement(XElement element) : base(element, cNamespace) { }

        /// <summary>String value of this elements tag in the XML schema.</summary>
        public override string Tag { get { return cTag; } }

        #region HideSystemFiles

        private const string cHideSystemFiles = "hideSystemFiles";

        /// <summary>Hide system file flag is changing.</summary>
        public event EventHandler HideSystemFilesChanging;
        /// <summary>Hide system file flag has changed.</summary>
        public event EventHandler HideSystemFilesChanged;

        /// <summary>Hide system files.</summary>
        /// <remarks>Configuration flag which determines whether system files will be shown in the list view.</remarks>
        public bool HideSystemFiles
        {
            get { return GetSimpleRequiredElementBool(cHideSystemFiles); }
            set { SetSimpleRequiredElement<bool>(cHideSystemFiles, value, HideSystemFilesChanging, HideSystemFilesChanged); }
        }

        #endregion

        #region ShowHiddenFiles

        private const string cShowHiddenFiles = "showHiddenFiles";

        /// <summary>Show hidden files flag is changing.</summary>
        public event EventHandler ShowHiddenFilesChanging;
        /// <summary>Show hidden files flag has changed.</summary>
        public event EventHandler ShowHiddenFilesChanged;

        public bool ShowHiddenFiles
        {
            get { return GetSimpleRequiredElementBool(cShowHiddenFiles); }
            set { SetSimpleRequiredElement<bool>(cShowHiddenFiles, value, ShowHiddenFilesChanging, ShowHiddenFilesChanged); }
        }

        #endregion

        #region HideExtensions

        private const string cHideExtensions = "hideExtensions";

        public event EventHandler HideExtensionsChanging;
        public event EventHandler HideExtensionsChanged;

        public bool HideExtensions
        {
            get { return GetSimpleRequiredElementBool(cHideExtensions); }
            set { SetSimpleRequiredElement<bool>(cHideExtensions, value, HideExtensionsChanging, HideExtensionsChanged); }
        }

        #endregion

        #region SaveAccount

        private const string cSaveAccount = "saveAccount";

        public event EventHandler SaveAccountChanging;
        public event EventHandler SaveAccountChanged;

        public bool SaveAccount
        {
            get { return GetSimpleRequiredElementBool(cSaveAccount); }
            set { SetSimpleRequiredElement<bool>(cSaveAccount, value, SaveAccountChanging, SaveAccountChanged); }
        }

        #endregion

        #region TargetServer

        private const string cTargetServer = "targetServer";

        /// <summary>Target server is changing.</summary>
        public event EventHandler TargetServerChanging;
        /// <summary>Target server has changed.</summary>
        public event EventHandler TargetServerChanged;

        public string TargetServer
        {
            get { return GetSimpleRequiredElementString(cTargetServer, string.Empty); }
            set { SetSimpleRequiredElement<string>(cTargetServer, value, TargetServerChanging, TargetServerChanged); }
        }

        /// <summary>Get target server.</summary>
        /// <returns>Server element containing the server information which is presently designated as the target server.</returns>
        public ApplicationData.ServerElement GetTargetServer() { return mServers[TargetServer]; }

        #endregion

        #region DefaultAccount

        private const string cDefaultAccount = "defaultAccount";

        public event EventHandler DefaultAccountChanging;
        public event EventHandler DefaultAccountChanged;

        public string DefaultAccount
        {
            get { return GetSimpleRequiredElementString(cDefaultAccount, string.Empty); }
            set { SetSimpleRequiredElement<string>(cDefaultAccount, value, DefaultAccountChanging, DefaultAccountChanged); }
        }

        #endregion

        #region Server

        private ApplicationData.ServerElementCollection mServers;

        public ApplicationData.ServerElementCollection Servers
        {
            get
            {
                if (mServers == null)
                {
                    mServers = new ApplicationData.ServerElementCollection(Element);

                    mServers.ItemRemoving += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.ServersEventArgs>(mServers_ItemRemoving);
                    mServers.ItemRemoved += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.ServersEventArgs>(mServers_ItemRemoved);
                }

                return mServers;
            }
        }

        void mServers_ItemRemoved(object sender, Sycamores.MobileShare.Client.ApplicationData.ServersEventArgs e)
        {
            throw new NotImplementedException();
        }

        void mServers_ItemRemoving(object sender, Sycamores.MobileShare.Client.ApplicationData.ServersEventArgs e)
        {
            throw new NotImplementedException();
        }

        #endregion

        #region Account

        private ApplicationData.AccountElementCollection mAccounts;

        public ApplicationData.AccountElementCollection Accounts
        {
            get
            {
                if (mAccounts == null)
                {
                    mAccounts = new ApplicationData.AccountElementCollection(Element);

                    mAccounts.ItemRemoving += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.AccountsEventArgs>(mAccounts_ItemRemoving);
                    mAccounts.ItemRemoved += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.AccountsEventArgs>(mAccounts_ItemRemoved);
                }

                return mAccounts;
            }
        }

        void mAccounts_ItemRemoved(object sender, Sycamores.MobileShare.Client.ApplicationData.AccountsEventArgs e)
        {
            if (DefaultAccount.Equals(e.Element.Id))
                DefaultAccount = string.Empty;
        }

        void mAccounts_ItemRemoving(object sender, Sycamores.MobileShare.Client.ApplicationData.AccountsEventArgs e)
        {
            if (TargetServer.Equals(e.Element.Id))
                TargetServer = string.Empty;
        }

        #endregion
    }

    namespace ApplicationData
    {
        public class ServerElement : ComplexType
        {
            private const string cTag = "server";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";

            public ServerElement(XElement element) : base(element, cNamespace) { }

            #region Id

            private const string cId = "id";
            private const string cIdDefault = null;

            public event EventHandler IdChanging;
            public event EventHandler IdChanged;

            public string Id
            {
                get { return GetRequiredAttributeString(cId, cIdDefault); }
                set { SetRequiredAttribute<string>(cId, value, IdChanging, IdChanged); }
            }

            #endregion

            #region Base

            private const string cBase = "base";

            public event EventHandler BaseChanging;
            public event EventHandler BaseChanged;

            public string Base
            {
                get { return GetSimpleRequiredElementString(cBase, string.Empty); }
                set { SetSimpleRequiredElement<string>(cBase, value, BaseChanging, BaseChanged); }
            }

            #endregion

            #region Address

            private const string cAddress = "address";

            public event EventHandler AddressChanging;
            public event EventHandler AddressChanged;

            public string Address
            {
                get { return GetSimpleOptionalElementString(cAddress); }
                set { SetSimpleOptionalElement<string>(cAddress, value, AddressChanging, AddressChanged); }
            }

            #endregion

            #region Port

            private const string cPort = "port";

            public event EventHandler PortAdding;
            public event EventHandler PortAdded;

            public event EventHandler PortChanging;
            public event EventHandler PortChanged;

            public event EventHandler PortRemoving;
            public event EventHandler PortRemoved;

            public int? Port
            {
                get { return GetSimpleOptionalElementInt(cPort); }
                set { SetSimpleOptionalElement<int?>(cPort, value, PortChanging, PortChanged); }
            }

            #endregion

            public override string Tag
            {
                get { return cTag; }
            }

            public override string ToString()
            {
                string address = string.Empty;
                if (Address != null) address = string.Concat("/", Address);

                string port = string.Empty;
                if (Port != null) port = string.Concat(":", Port.ToString());

                return string.Concat("http://", Base, port, address);
            }

            public Uri ToUri() { return new Uri(ToString()); }

            public override bool Equals(object obj)
            {
                ServerElement external = obj as ServerElement;

                if (obj != null)
                {
                    return Id.Equals(external.Id)
                        && Base.Equals(external.Base)
                        && Address.Equals(external.Address)
                        && Port.Equals(external.Port);
                }
                else
                    return false;
            }

            public override int GetHashCode()
            {
                return base.GetHashCode();
            }
        }

        public class ServerElementCollection : KeyedComplexTypeCollection<string, ServerElement, ServersEventArgs>
        {
            private const string cTag = "server";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";
            private const string cId = "id";

            protected internal ServerElementCollection(XElement element)
                : base(cNamespace, element)
            {
                foreach (var item in Element.Elements(GetElementName(cTag)))
                {
                    Add(item.Attribute(cId).Value, new ServerElement(item));
                }
            }

            public ServerElement GetNewServer()
            {
                return new ServerElement(new XElement(GetElementName(cTag)));
            }

            public void Add(ServerElement value)
            {
                Dictionary.Add(value.Id, value);

                Add(GetElementName(cTag), value);
            }

            public override string Tag
            {
                get { return cTag; }
            }

            public override void Add(XName tag, ServerElement value)
            {
                if (Element.Elements(tag).Count() > 0)
                    Element.Elements(tag).Last().AddAfterSelf(value.Element);
                else
                    Element.Add(value.Element);
            }
        }

        public class AccountElement : ComplexType
        {
            private const string cTag = "account";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";

            public AccountElement(XElement element) : base(element, cNamespace) { }

            #region Id

            private const string cId = "id";
            private const string dId = null;

            public event EventHandler IdChanging;
            public event EventHandler IdChanged;

            public string Id
            {
                get { return GetRequiredAttributeString(cId, dId); }
                set { SetRequiredAttribute<string>(cId, value, IdChanging, IdChanged); }
            }

            #endregion

            #region Cookie

            private const string cCookie = "cookie";

            public event EventHandler CookieChanging;
            public event EventHandler CookieChanged;

            public string Cookie
            {
                get { return GetSimpleRequiredElementString(cCookie, string.Empty); }
                set { SetSimpleRequiredElement<string>(cCookie, value, CookieChanging, CookieChanged); }
            }

            #endregion

            #region Resource

            public ApplicationData.Account.ResourceElementCollection mResources;

            public ApplicationData.Account.ResourceElementCollection Resources
            {
                get
                {
                    if (mResources == null)
                    {
                        mResources = new Account.ResourceElementCollection(Element);

                        mResources.ItemRemoving += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.Account.ResourcesEventArgs>(mResources_ItemRemoving);
                        mResources.ItemRemoved += new EventHandler<Sycamores.MobileShare.Client.ApplicationData.Account.ResourcesEventArgs>(mResources_ItemRemoved);
                    }

                    return mResources;
                }
            }

            void mResources_ItemRemoved(object sender, Sycamores.MobileShare.Client.ApplicationData.Account.ResourcesEventArgs e)
            {
                throw new NotImplementedException();
            }

            void mResources_ItemRemoving(object sender, Sycamores.MobileShare.Client.ApplicationData.Account.ResourcesEventArgs e)
            {
                throw new NotImplementedException();
            }

            #endregion

            public override string Tag
            {
                get { return cTag; }
            }
        }

        public class AccountElementCollection : KeyedComplexTypeCollection<string, AccountElement, AccountsEventArgs>
        {
            private const string cTag = "account";
            private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";
            private const string cId = "id";

            protected internal AccountElementCollection(XElement element)
                : base(cNamespace, element)
            {
                lock (Locker)
                {
                    foreach (var item in element.Elements(GetElementName(cTag)))
                    {
                        Add(item.Attribute(cId).Value, new AccountElement(item));
                    }
                }
            }

            public AccountElement GetNewAccount()
            {
                return new AccountElement(new XElement(GetElementName(cTag)));
            }

            public void Add(AccountElement value)
            {
                lock (Locker)
                {
                    Dictionary.Add(value.Id, value);
                    Add(GetElementName(cTag), value);
                    value.IdChanged += new EventHandler(value_IdChanged);
                    value.IdChanging += new EventHandler(value_IdChanging);
                }
            }

            public new event EventHandler<AccountsEventArgs> ItemRemoving;
            public new event EventHandler<AccountsEventArgs> ItemRemoved;

            public new bool Remove(string key)
            {
                lock (Locker)
                {
                    if (ItemRemoving != null) ItemRemoving(this, new AccountsEventArgs(this[key]));

                    this[key].IdChanged -= new EventHandler(value_IdChanged);
                    this[key].IdChanging -= new EventHandler(value_IdChanging);

                    var query = from account in Element.Elements(GetElementName(cTag))
                                where account.Attributes(GetAttributeName("id")).First().Value.Equals(key)
                                select account;

                    foreach (XElement item in query)
                        item.Remove();

                    if (ItemRemoved != null) ItemRemoved(this, new AccountsEventArgs(this[key]));
                }

                return Dictionary.Remove(key);
            }

            void value_IdChanging(object sender, EventArgs e)
            {
                throw new NotImplementedException();
            }

            void value_IdChanged(object sender, EventArgs e)
            {
                throw new NotImplementedException();
            }

            public override string Tag
            {
                get { return cTag; }
            }

            public override void Add(XName tag, AccountElement value)
            {
                if (Element.Elements(cTag).Count() > 0)
                    Element.Elements(cTag).Last().AddAfterSelf(value.Element);
                else
                    Element.Elements(GetElementName("defaultAccount")).First().AddAfterSelf(value.Element);
            }
        }

        public class ServersEventArgs : EventArgs
        {
            private ServerElement mElement;

            public ServersEventArgs(ServerElement element)
            {
                mElement = element;
            }

            public ServerElement Element { get { return mElement; } }
        }

        public class AccountsEventArgs : EventArgs
        {
            private AccountElement mElement;

            public AccountsEventArgs(AccountElement element)
            {
                mElement = element;
            }

            public AccountElement Element { get { return mElement; } }
        }

        namespace Account
        {
            public class ResourceElementCollection : ComplexTypeCollection<ResourceElement, ResourcesEventArgs>
            {
                private const string cTag = "account";
                private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";
                private const string cId = "id";

                protected internal ResourceElementCollection(XElement element)
                    : base(cNamespace, element)
                {
                    lock (Locker)
                    {
                        foreach (var item in element.Elements(GetElementName(cTag)))
                        {
                            Add(item.Attribute(cId).Value, new ResourceElement(item));
                        }
                    }
                }

                public ResourceElement GetNewResource()
                {
                    return new ResourceElement(new XElement(GetElementName(cTag)));
                }

                public new void Add(ResourceElement value)
                {
                    lock (Locker)
                    {
                        List.Add(value);
                        Add(GetElementName(cTag), value);
                    }
                }

                public bool Remove(string key)
                {
                    //lock (Locker)
                    //{
                    //    if (ItemRemoving != null) ItemRemoving(this, new ResourcesEventArgs(this[key]));

                    //    this[key].IdChanged -= new EventHandler(value_IdChanged);
                    //    this[key].IdChanging -= new EventHandler(value_IdChanging);

                    //    var query = from account in Element.Elements(GetElementName(cTag))
                    //                where account.Attributes(GetAttributeName("id")).First().Value.Equals(key)
                    //                select account;

                    //    foreach (XElement item in query)
                    //        item.Remove();

                    //    if (ItemRemoved != null) ItemRemoved(this, new ResourcesEventArgs(this[key]));
                    //}

                    //return Dictionary.Remove(key);

                    throw new NotImplementedException();
                }

                void value_IdChanging(object sender, EventArgs e)
                {
                    throw new NotImplementedException();
                }

                void value_IdChanged(object sender, EventArgs e)
                {
                    throw new NotImplementedException();
                }

                public override string Tag
                {
                    get { return cTag; }
                }

                public override void Add(XName tag, ResourceElement value)
                {
                    if (Element.Elements(cTag).Count() > 0)
                        Element.Elements(cTag).Last().AddAfterSelf(value.Element);
                    else
                        Element.Elements(GetElementName("defaultResource")).First().AddAfterSelf(value.Element);
                }
            }

            public class ResourceElement : ComplexType
            {
                private const string cTag = "resource";
                private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";

                public ResourceElement(XElement element) : base(element, cNamespace) { }

                public override string Tag { get { return cTag; } }

                #region Domain

                private const string cDomain = "domain";

                public event EventHandler DomainChanging;
                public event EventHandler DomainChanged;

                public bool Domain
                {
                    get { return GetSimpleRequiredElementBool(cDomain); }
                    set { SetSimpleRequiredElement<bool>(cDomain, value, DomainChanging, DomainChanged); }
                }

                #endregion

                #region Computer

                private const string cComputer = "computer";

                public event EventHandler ComputerChanging;
                public event EventHandler ComputerChanged;

                public bool Computer
                {
                    get { return GetSimpleRequiredElementBool(cComputer); }
                    set { SetSimpleRequiredElement<bool>(cComputer, value, ComputerChanging, ComputerChanged); }
                }

                #endregion

                #region Share

                private const string cShare = "share";

                public event EventHandler ShareChanging;
                public event EventHandler ShareChanged;

                public bool Share
                {
                    get { return GetSimpleRequiredElementBool(cShare); }
                    set { SetSimpleRequiredElement<bool>(cShare, value, ShareChanging, ShareChanged); }
                }

                #endregion
            }

            public class ResourcesEventArgs : EventArgs
            {
                private ResourceElement mElement;

                public ResourcesEventArgs(ResourceElement element)
                {
                    mElement = element;
                }

                public ResourceElement Element { get { return mElement; } }
            }
        }
    }
}