﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Schema;

using Sycamores.XFrameworkCF;

namespace Sycamores.MobileShare.Client
{
    public class ApplicationDataFile : ValidatingXmlFile
    {
        private const string cRootTag = "applicationData";
        //private const string cNamespace = "http://janglin.com/Sycamores/MobileShare/Client/applicationData";

        private ApplicationDataFile(XmlSchema schema) :
            base(AppDataFilePath, schema, GetDefaultRoot())
        {
            Filename = AppDataFilePath;
        }

        private static string AppDataFilePath { get { return GetFullAppDataPath("ApplicationData.xml"); } }

        private static XDocument GetDefaultRoot()
        {
            return ValidatingXmlFile.GetEmbeddedResourceXDoc("Sycamores.MobileShare.Client.ApplicationData.ApplicationData.xml");
        }

        public override void Save() { Document.Save(Filename); }

        public static void Purge()
        {
            string folder = AppDataFolder;

            if (File.Exists(GetFullAppDataPath("ApplicationData.xsd")))
                File.Delete(GetFullAppDataPath("ApplicationData.xsd"));

            if (File.Exists(GetFullAppDataPath("ApplicationData.xml")))
                File.Delete(GetFullAppDataPath("ApplicationData.xml"));

            if (File.Exists(GetFullAppDataPath("ApplicationData.xslt")))
                File.Delete(GetFullAppDataPath("ApplicationData.xslt"));
        }

        private static ApplicationDataFile mFile;

        public static ApplicationDataFile GetFile()
        {
            if (ApplicationDataFile.mFile == null)
            {
                const string xsdresourcename = "Sycamores.MobileShare.Client.ApplicationData.ApplicationData.xsd";
                const string xsltresourcename = "Sycamores.MobileShare.Client.ApplicationData.ApplicationData.xslt";

                XmlSchema schema = ValidatingXmlFile.GetEmbeddedResourceSchema(xsdresourcename);

                string xsltpath = GetFullAppDataPath("ApplicationData.xslt");

                if (!File.Exists(xsltpath))
                    ValidatingXmlFile.WriteEmbeddedResourceXml(xsltresourcename, xsltpath);

                ApplicationDataFile.mFile = new ApplicationDataFile(schema);
            }

            return ApplicationDataFile.mFile;
        }

        private static string AppDataFolder
        {
            get
            {
                const string thisappdatafolder= "Sycamores\\MobileShare\\Client\\";
                string appdatafolder = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);

                return Path.Combine(appdatafolder, thisappdatafolder);
            }
        }

        private static string GetFullAppDataPath(string filename)
        {
            return Path.Combine(AppDataFolder, filename);
        }

        public override void UpdateFile()
        {
            if (Document.Root.Attributes("version").First().Value.Equals("1.0.1"))
            {
                string xsltpath = GetFullAppDataPath("ApplicationData1_0_3.xslt");
                const string xsltresourcename = "Sycamores.MobileShare.Client.ApplicationData.ApplicationData1_0_3.xslt";

                string xslt = ValidatingXmlFile.GetEmbeddedResourceFileXmlText(xsltresourcename);

                UpdateFile(GetFullAppDataPath("ApplicationData.xml"), xslt);

                if (File.Exists(xsltpath)) File.Delete(xsltpath);

                xsltpath = GetFullAppDataPath("ApplicationData.xslt");
                const string xslthtmlresourcename = "Sycamores.MobileShare.Client.ApplicationData.ApplicationData.xslt";
                ValidatingXmlFile.WriteEmbeddedResourceXml(xslthtmlresourcename, xsltpath);
            }
        }

        #region ApplicationDataElement

        private ApplicationDataElement mApplicationDataElement;

        public ApplicationDataElement ApplicationDataElement
        {
            get
            {
                if (mApplicationDataElement == null)
                    mApplicationDataElement = new ApplicationDataElement(Document.Root);

                return mApplicationDataElement;
            }
        }

        #endregion
    }
}