﻿using System;
using System.Windows.Forms;
using System.Collections.Generic;
using System.IO;

using Sycamores.PlatformInvokeCF.Gwes;
using SysInfo = Sycamores.PlatformInvokeCF.SystemServices.SystemInformation;

namespace Sycamores.MobileShare.Client
{
    /// <summary>ListView with a label control as a banner.</summary>
    /// <remarks>
    /// <list type="table">
    ///   <listheader>
    ///      <term>Pocket PC</term>
    ///       <description>(normal DPI)</description>
    ///   </listheader>
    ///   <item>
    ///       <term>DPI</term>
    ///       <description>96</description>
    ///    </item>
    ///   <item>
    ///       <term>Small Icons</term>
    ///       <description>16x16</description>
    ///    </item>
    ///   <item>
    ///       <term>Large Icons</term>
    ///       <description>32x32</description>
    ///    </item>
    ///   <listheader>
    ///      <term>Pocket PC</term>
    ///       <description>(high DPI)</description>
    ///   </listheader>
    ///   <item>
    ///       <term>DPI</term>
    ///       <description>192</description>
    ///    </item>
    ///   <item>
    ///       <term>Small Icons</term>
    ///       <description>32x32</description>
    ///    </item>
    ///   <item>
    ///       <term>Large Icons</term>
    ///       <description>64x64</description>
    ///    </item>
    ///   <listheader>
    ///      <term>Smartphone</term>
    ///       <description>(normal DPI)</description>
    ///   </listheader>
    ///   <item>
    ///       <term>DPI</term>
    ///       <description>96</description>
    ///    </item>
    ///   <item>
    ///       <term>Small Icons</term>
    ///       <description>16x16</description>
    ///    </item>
    ///   <item>
    ///       <term>Large Icons</term>
    ///       <description>32x32</description>
    ///    </item>
    ///   <listheader>
    ///      <term>Smartphone</term>
    ///       <description>(high DPI)</description>
    ///   </listheader>
    ///   <item>
    ///       <term>DPI</term>
    ///       <description>131</description>
    ///    </item>
    ///   <item>
    ///       <term>Small Icons</term>
    ///       <description>22x22</description>
    ///    </item>
    ///   <item>
    ///       <term>Large Icons</term>
    ///       <description>44x44</description>
    ///    </item>
    /// </list>
    ///</remarks>
    public partial class BanneredListView : UserControl
    {
        public event EventHandler ItemActivate;
        public event EventHandler SelectedIndexChanged;

        /// <summary>Default constructor.</summary>
        public BanneredListView()
        {
            InitializeComponent();

            this.ListView.View = View.LargeIcon;

            switch (SysInfo.GetIconSizeLarge().Width)
            {
                case 44:
                    ListView.LargeImageList = Icon44List;
                    break;
                default:
                    ListView.LargeImageList = Icon32List;
                    break;
            }

            switch (SysInfo.GetIconSizeSmall().Width)
            {
                case 22:
                    ListView.SmallImageList = Icon22List;
                    break;
                default:
                    ListView.SmallImageList = Icon16List;
                    break;
            }
        }

        /// <summary>Contained listview control.</summary>
        public ListView ListView { get { return this.ListViewCtrl; } }

        public override string Text
        {
            get { return BannerLabel.Text; }
            set
            {
                BannerLabel.ForeColor = System.Drawing.Color.Black;
                BannerLabel.Text = value;
            }
        }

        public void SetRedText(string value)
        {
            BannerLabel.ForeColor = System.Drawing.Color.Red;
            BannerLabel.Text = string.Concat("<", value, ">");
        }

        public void AddListViewItemDomain(string domain)
        {
            ListViewItem listitem = ListViewCtrl.Items.Add(new ListViewItem(domain));
            listitem.ImageIndex = 0;
            listitem.Tag = domain;
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem();
            subitem.Text = Properties.Resources.SubItemWorkgroup;
            listitem.SubItems.Add(subitem);
        }

        public void AddListViewItemComputer(string computer)
        {
            ListViewItem listitem = ListViewCtrl.Items.Add(new ListViewItem(computer));
            listitem.ImageIndex = 1;
            listitem.Tag = computer;
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem();
            subitem.Text = Properties.Resources.SubItemComputer;
            listitem.SubItems.Add(subitem);
        }

        public void AddListViewItemShares(string item)
        {
            ListViewItem listitem = ListViewCtrl.Items.Add(new ListViewItem());
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem();

            if (item.Equals("print$"))
            {
                listitem.ImageIndex = 3;
                listitem.Text = Properties.Resources.ListItemPrinters;
                subitem.Text = Properties.Resources.SubItemSystemFolder;
            }
            else
            {
                listitem.ImageIndex = 2;
                listitem.Text = item;
                subitem.Text = Properties.Resources.SubItemShare;
            }

            listitem.Tag = item;
            listitem.SubItems.Add(subitem);
        }

        public ListViewItem AddListViewItemDirectory(DirectoryInfo info)
        {
            ListViewItem item = ListViewCtrl.Items.Add(new ListViewItem(info.Name));
            item.ImageIndex = 6;
            item.Tag = info;
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem();
            subitem.Text = Properties.Resources.SubItemFileFolder;
            item.SubItems.Add(subitem);

            return item;
        }

        public void AddListViewItemFile(FileInfo info)
        {
            string extension = info.Extension.ToUpper();

            ListViewItem listitem = ListViewCtrl.Items.Add(new ListViewItem(info.Name));
            ListViewItem.ListViewSubItem subitem = new ListViewItem.ListViewSubItem();
            listitem.ImageIndex = GetIcon(extension);

            if (extension.Equals(".EXE"))
            {
                listitem.ImageIndex = 7;
                subitem.Text = Properties.Resources.SubItemApplication;
            }
            else if (extension.Equals(".DLL"))
            {
                listitem.ImageIndex = 8;
                subitem.Text = Properties.Resources.SubItemDll;
            }
            else if (mIconDictionary.ContainsKey(extension))
            {
                listitem.ImageIndex = mIconDictionary[extension];
                subitem.Text = GetFileType(extension);
            }
            else
            {
                listitem.ImageIndex = 9;
                subitem.Text = string.Concat(extension.ToUpper(), Properties.Resources.SubItemFileSuffix);
            }

            listitem.Tag = info;
            listitem.SubItems.Add(subitem);
        }

        private string GetFileType(string ext)
        {
            if (!mFileTypeDictionary.ContainsKey(ext))
            {
                RegistryEx key = new RegistryEx(ext);

                if (!string.IsNullOrEmpty(key.FileDescription))
                    mFileTypeDictionary.Add(ext.ToUpper(), key.FileDescription);
                else
                    return string.Concat(ext.ToUpper(), Properties.Resources.SubItemFileSuffix);
            }

            return mFileTypeDictionary[ext];
        }

        private int GetIcon(string ext)
        {
            if (!mIconDictionary.ContainsKey(ext))
            {
                RegistryEx key = new RegistryEx(ext);

                if (key.HasDefaultIcon)
                {
                    System.Drawing.Icon large = Resources.ExtractIconFromExe(Path.Combine("\\Windows", key.IconFile), key.IconIndex, true);
                    System.Drawing.Icon small = Resources.ExtractIconFromExe(Path.Combine("\\Windows", key.IconFile), key.IconIndex, false);

                    this.ListView.SmallImageList.Images.Add(small);
                    this.ListView.LargeImageList.Images.Add(large);

                    mIconDictionary.Add(ext.ToUpper(), this.ListView.LargeImageList.Images.Count - 1);
                }
                else
                    return 9;
            }

            return mIconDictionary[ext];
        }

        private IDictionary<string, int> mIconDictionary = new Dictionary<string, int>();
        private IDictionary<string, string> mFileTypeDictionary = new Dictionary<string, string>();

        private void BanneredListView_Resize(object sender, EventArgs e)
        {
            ListViewCtrl.Columns[0].Width = 100;
            ListViewCtrl.Columns[1].Width = ListViewCtrl.Width - 101;
        }

        private void ListViewCtrl_ItemActivate(object sender, EventArgs e)
        {
            if (ItemActivate != null) ItemActivate(sender, e);
        }

        private void ListViewCtrl_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (SelectedIndexChanged != null) SelectedIndexChanged(sender, e);
        }
    }
}