﻿using System;
using System.Windows.Forms;

namespace Sycamores.MobileShare.Client
{
    /// <summary>Item editing form.</summary>
    public partial class EditForm : Form
    {
        /// <summary>Default constructor.</summary>
        public EditForm() { InitializeComponent(); }

        /// <summary>Done.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected virtual void DoneMenuItem_Click(object sender, EventArgs e) { }

        private void CancelMenuItem_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }

        /// <summary>Return empty if input is null.</summary>
        /// <param name="input">Input value.</param>
        /// <returns>Returns empty string value if input is null or empty. Otherwise it returns the input value.</returns>
        protected internal static string EmptyIfNull(string input)
        {
            if (string.IsNullOrEmpty(input))
                return string.Empty;
            else
                return input;
        }

        /// <summary>Return empty if input is null.</summary>
        /// <param name="input">Input value.</param>
        /// <returns>Returns empty string value if input is null or empty. Otherwise it returns the input value as a string.</returns>
        protected internal static string EmptyIfNull(int? input)
        {
            if (!input.HasValue)
                return string.Empty;
            else
                return input.Value.ToString();
        }

        /// <summary>Return null if input is null or empty.</summary>
        /// <param name="input">Input value.</param>
        /// <returns>Returns null value if input is null or empty. Otherwise it returns the input value as a string.</returns>
        protected internal static string NullIfEmpty(string input)
        {
            if (string.IsNullOrEmpty(input))
                return null;
            else
                return input;
        }

        /// <summary>Return null if input is null or empty.</summary>
        /// <param name="input">Input value.</param>
        /// <returns>Returns null value if input is null or empty. Otherwise it returns the input value as a string.</returns>
        protected internal static int? NullIfEmptyInt(string input)
        {
            if (string.IsNullOrEmpty(input))
                return null;
            else
                return Convert.ToInt32(input);
        }
    }
}