﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Sycamores.MobileShare.Client
{
    public partial class FilePropertiesForm : Form
    {
        FileInfo mFileInfo;

        public FilePropertiesForm()
        {
            InitializeComponent();
        }

        public FilePropertiesForm(FileInfo fileInfo)
            : this()
        {
            mFileInfo = fileInfo;
        }

        private void FilePropertiesForm_Load(object sender, EventArgs e)
        {
            NameTextBox.Text = mFileInfo.Name;
            LocationTextBox.Text = mFileInfo.DirectoryName;
            TypeTextBox.Text = mFileInfo.Extension;
            SizeTextBox.Text = mFileInfo.Length.ToString();
            DateModifiedTextBox.Text = mFileInfo.LastWriteTime;
            DateCreatedTextBox.Text = mFileInfo.CreationTime;

            if (mFileInfo.IsReadOnlySpecified)
                IsReadOnlyCheckBox.Checked = mFileInfo.IsReadOnly;
            else
                IsReadOnlyCheckBox.CheckState = CheckState.Indeterminate;

            IsReadOnlyCheckBox_CheckStateChanged(this, EventArgs.Empty);
        }

        private void OkMenuItem_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
        }

        private void IsReadOnlyCheckBox_CheckStateChanged(object sender, EventArgs e)
        {
            if (mFileInfo.AttributesSpecified)
            {
                System.IO.FileAttributes attributes = (System.IO.FileAttributes)mFileInfo.Attributes;

                HiddenCheckBox.Checked = (System.IO.FileAttributes.Hidden & attributes) != 0;
                ArchiveCheckBox.Checked = (System.IO.FileAttributes.Archive & attributes) != 0;
                CompressedCheckBox.Checked = (System.IO.FileAttributes.Compressed & attributes) != 0;
                EncryptedCheckBox.Checked = (System.IO.FileAttributes.Encrypted & attributes) != 0;
            }
            else
            {
                HiddenCheckBox.CheckState = CheckState.Indeterminate;
                ArchiveCheckBox.CheckState = CheckState.Indeterminate;
                CompressedCheckBox.CheckState = CheckState.Indeterminate;
                EncryptedCheckBox.CheckState = CheckState.Indeterminate;
            }
        }
    }
}