﻿using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace Sycamores.MobileShare.Client
{
    /// <summary>Edit a server connection item.</summary>
    public partial class ServerEditForm : EditForm
    {
        /// <summary>Default constructor.</summary>
        public ServerEditForm() { InitializeComponent(); }

        /// <summary>Populate with application data.</summary>
        /// <param name="serverData">Application data.</param>
        public ServerEditForm(ApplicationData.ServerElement serverData)
            : this()
        {
            mServerData = serverData;
        }

        private ApplicationData.ServerElement mServerData;

        private void ServerEditForm_Load(object sender, EventArgs e)
        {
            DescriptionTextBox.Text = mServerData.Id;
            ServerTextBox.Text = mServerData.Base;
            AddressTextBox.Text = EmptyIfNull(mServerData.Address);
            PortTextBox.Text = EmptyIfNull(mServerData.Port);
        }

        /// <summary>Done.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected override void DoneMenuItem_Click(object sender, EventArgs e)
        {
            mServerData.Id = DescriptionTextBox.Text;
            mServerData.Base = ServerTextBox.Text;
            mServerData.Address = NullIfEmpty(AddressTextBox.Text);
            mServerData.Port = NullIfEmptyInt(PortTextBox.Text);

            this.DialogResult = DialogResult.OK;
        }

        private void ServerEditForm_Closing(object sender, CancelEventArgs e)
        {
            if (!DialogResult.Equals(DialogResult.Cancel)
                && (DescriptionTextBox.Text.Length <= 0 || ServerTextBox.Text.Length <= 0))
            {
                MessageBox.Show(Properties.Resources.MessageServerValidate
                    , Properties.Resources.CaptionServerEditValidate
                    , MessageBoxButtons.OK
                    , MessageBoxIcon.Hand
                    , MessageBoxDefaultButton.Button1);

                e.Cancel = true;
            }
        }
    }
}