﻿using System;
using System.Windows.Forms;

using Res = Sycamores.MobileShare.Client.Properties.Resources;

namespace Sycamores.MobileShare.Client
{
    /// <summary>Form allowing user to add, edit and delete server connections.</summary>
    public partial class ServerMenuForm : MenuForm
    {
        /// <summary>Default constructor.</summary>
        public ServerMenuForm()
            : base(true)
        {
            InitializeComponent();
        }

        /// <summary>Populate form according to available data.</summary>
        /// <param name="appData">Application data.</param>
        public ServerMenuForm(ApplicationDataElement appData)
            : base(true)
        {
            InitializeComponent();

            mAppData = appData;
        }

        private ApplicationDataElement mAppData;

        private void ServerMenuForm_Load(object sender, EventArgs e)
        {
            Clear();

            if (mAppData.Servers.Count <= 0)
            {
                AddMenuItem_Click(this, EventArgs.Empty);

                DoneMenuItem_Click(this, EventArgs.Empty);
            }
            else
            {
                foreach (ApplicationData.ServerElement server in mAppData.Servers.Values)
                    Add(server.Id);
            }
        }

        /// <summary>Item selected.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected override void ItemSelected()
        {
            if (SelectedValue.Equals(Res.MenuNewValue))
                AddMenuItem_Click(this, EventArgs.Empty);
            else
                EditMenuItem_Click(this, EventArgs.Empty);
        }

        /// <summary>Add item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected override void AddMenuItem_Click(object sender, EventArgs e)
        {
            ShowEditDialog(mAppData, mAppData.Servers.GetNewServer());
            ServerMenuForm_Load(this, EventArgs.Empty);
        }

        /// <summary>Edit item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected override void EditMenuItem_Click(object sender, EventArgs e)
        {
            ShowEditDialog(mAppData, mAppData.Servers[SelectedValue]);
            ServerMenuForm_Load(this, EventArgs.Empty);
        }

        /// <summary>Delete item.</summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        protected override void DeleteMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(Res.MessageDeleteServer
                , Res.CaptionDeleteServer
                , MessageBoxButtons.YesNo
                , MessageBoxIcon.Question
                , MessageBoxDefaultButton.Button2).Equals(DialogResult.Yes))
            {
                mAppData.Servers.Remove(SelectedValue);

                RemoveSelection();
            }
        }

        internal static void ShowEditDialog(ApplicationDataElement appData,ApplicationData.ServerElement server)
        {
            using (ServerEditForm form = new ServerEditForm(server))
            {
                if (form.ShowDialog().Equals(DialogResult.OK))
                {
                    if (!appData.Servers.ContainsKey(server.Id)) appData.Servers.Add(server);

                    if (appData.TargetServer.Equals(server.Id)) return;

                    if (appData.Servers.Count.Equals(1))
                    {
                        MessageBox.Show(string.Format(Res.MessageEditServer1, server.Id)
                            , Res.CaptionEditServer
                            , MessageBoxButtons.OK
                            , MessageBoxIcon.Asterisk
                            , MessageBoxDefaultButton.Button1);

                        appData.TargetServer = server.Id;
                    }
                    else
                    {
                        if (MessageBox.Show(string.Format(Res.MessageEditServerN, server.Id)
                            , Res.CaptionEditServer
                            , MessageBoxButtons.YesNo
                            , MessageBoxIcon.Question
                            , MessageBoxDefaultButton.Button1).Equals(DialogResult.Yes))
                        {
                            appData.TargetServer = server.Id;
                        }
                    }
                }
            }
        }
    }
}
