﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.Linq;
using System.IO;
using Microsoft.Win32;

namespace ActiveSyncInstallation
{
    [RunInstaller(true)]
    public partial class ActiveSyncInstaller : Installer
    {
        private const string cAppMgrExe = @"SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\CEAPPMGR.EXE";
        private const string cActiveSyncInstallPath = @"SOFTWARE\Microsoft\Windows CE Services";
        private const string cInstalledDir = "InstalledDir";
        private const string cAppMgrExeFile = @"CEAPPMGR.EXE";
        private const string cCeAppMgrIniFile = @"MobileShare.ini";
        private const string cAppSubDir = @"\MobileShare";

        private string mTempPath = Path.Combine(Environment.SystemDirectory, @"Temp\MobileShare");

        public ActiveSyncInstaller()
        {
            InitializeComponent();

            this.BeforeInstall += new InstallEventHandler(ActiveSyncInstaller_BeforeInstall);
            this.AfterInstall += new InstallEventHandler(ActiveSyncInstaller_AfterInstall);
            this.BeforeUninstall += new InstallEventHandler(ActiveSyncInstaller_BeforeUninstall);
        }

        private string GetAppInstallDirectory()
        {
            // Get the ActiveSync install directory
            RegistryKey activesynckey = Registry.LocalMachine.OpenSubKey(cActiveSyncInstallPath);

            if (activesynckey == null)
                return null;

            // Build the target directory path under the ActiveSync folder
            string activesyncpath = (string)activesynckey.GetValue(cInstalledDir);
            string installpath = string.Concat(activesyncpath, cAppSubDir);

            activesynckey.Close();

            return installpath;
        }

        void ActiveSyncInstaller_BeforeInstall(object sender, InstallEventArgs e)
        {
            const string bslash = "\"";

            // Find the location where the application will be installed
            string installpath = GetAppInstallDirectory();

            if (installpath == null)
            {
                switch (
            System.Windows.Forms.MessageBox.Show(string.Concat("ActiveSync is not installed. The Mobile Share Client application will have to be manually installed on the mobile device.")
            , "Mobile Share Personal Edition"
            , System.Windows.Forms.MessageBoxButtons.OKCancel
            , System.Windows.Forms.MessageBoxIcon.Exclamation
            , System.Windows.Forms.MessageBoxDefaultButton.Button1))
                {
                    case System.Windows.Forms.DialogResult.Cancel:
                        throw new InvalidOperationException("ActiveSync is not installed. Installation cancelled.");
                    //case System.Windows.Forms.DialogResult.OK:
                    //    return;
                }

                return;
            }

            // Create the target directory
            Directory.CreateDirectory(installpath);
            // Copy your application files to the directory
            foreach (string installfile in Directory.GetFiles(mTempPath))
            {
                File.Copy(installfile, Path.Combine(installpath, Path.GetFileName(installfile)), true);
            }

            // Get the path to ceappmgr.exe
            RegistryKey appmgrkey = Registry.LocalMachine.OpenSubKey(cAppMgrExe);

            string appmgrpath = (string)appmgrkey.GetValue(null);
            appmgrkey.Close();

            // Run CeAppMgr.exe to install the files to the device
            string start = string.Concat(bslash, Path.Combine(installpath, cCeAppMgrIniFile), bslash);

            System.Diagnostics.Process.Start(appmgrpath, start);
        }

        void ActiveSyncInstaller_AfterInstall(object sender, InstallEventArgs e)
        {
            // Delete the temp files
            foreach (string tempFile in Directory.GetFiles(mTempPath))
            {
                File.Delete(tempFile);
            }
        }

        void ActiveSyncInstaller_BeforeUninstall(object sender, InstallEventArgs e)
        {
            // Find where the application is installed
            string installpath = GetAppInstallDirectory();

            if (installpath == null)
                return;

            // Delete the files
            foreach (string appfile in Directory.GetFiles(installpath))
            {
                File.Delete(appfile);
            }

            // Delete the folder
            Directory.Delete(installpath);
        }

        internal static void Log(string message)
        {
            using (System.Diagnostics.EventLog appLog = new System.Diagnostics.EventLog())
            {
                appLog.Source = "ActiveSync Installer";
                appLog.WriteEntry(message, System.Diagnostics.EventLogEntryType.Error);
            }
        }
    }
}