﻿using System;
using System.Runtime.InteropServices;

namespace Sycamores.PlatformInvokeCF.Diagnostics
{
    /// <summary>Error handling.</summary>
    /// <remarks>Well-written applications include error-handling code that allows them to recover gracefully from unexpected errors. 
    /// When an error occurs, the application may need to request user intervention, or it may be able to recover on its own. 
    /// In extreme cases, the application may log the user off or shut down the system.</remarks>
    public class ErrorHandling
    {
        [DllImport("coredll.dll", CharSet = System.Runtime.InteropServices.CharSet.Auto)]
        internal unsafe static extern int FormatMessage(int dwFlags
            , ref IntPtr lpSource
            , int dwMessageId
            , int dwLanguageId
            , ref String lpBuffer
            , int nSize
            , IntPtr* Arguments);

        internal unsafe static String GetLastErrorMessage()
        {
            const int FORMAT_MESSAGE_ALLOCATE_BUFFER = 0x00000100;
            const int FORMAT_MESSAGE_IGNORE_INSERTS = 0x00000200;
            const int FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000;

            String lpMsgBuf = string.Empty;
            int dwFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS;
            IntPtr ptrlpSource = new IntPtr();
            int errorcode = Marshal.GetLastWin32Error();

            int retVal = ErrorHandling.FormatMessage(dwFlags, ref ptrlpSource, errorcode, 0, ref lpMsgBuf, 0, null);

            if (retVal.Equals(0)) throw new InvalidOperationException(string.Concat("Failed to format message for error code ", errorcode, "."));

            return lpMsgBuf;
        }
    }
}