﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Xml.Linq;

namespace Sycamores.XFrameworkCF
{
    public abstract class ComplexType : Entity
    {
        private XElement mElement;

        protected ComplexType(XElement element, string nameSpace)
            : base(nameSpace)
        {
            if (element == null) throw new ArgumentNullException("element", "Element is null.");

            mElement = element;
        }

        public XElement Element { get { return mElement; } }

        protected string GetRequiredAttributeString(string tag, string defaultValue)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (Element.Attribute(GetAttributeName(tag)) == null)
            {
                if (defaultValue == null) defaultValue = string.Empty;

                Element.Add(new XAttribute(GetAttributeName(tag), defaultValue));
            }

            return Element.Attribute(GetAttributeName(tag)).Value;
        }

        protected void SetRequiredAttribute<T>(string tag, T value, EventHandler eventChanging, EventHandler eventChanged)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (!value.ToString().Equals(Element.Attribute(GetAttributeName(tag)).ToString()))
            {
                if (eventChanging != null) eventChanging(this, EventArgs.Empty);

                Element.SetAttributeValue(GetAttributeName(tag), value);

                if (eventChanged != null) eventChanged(this, EventArgs.Empty);
            }
        }

        protected string GetSimpleRequiredElementString(string tag, string defaultValue)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (Element.Element(GetElementName(tag)) == null)
            {
                if (defaultValue == null) defaultValue = string.Empty;

                Element.Add(new XElement(GetElementName(tag), defaultValue));
            }

            return Element.Element(GetElementName(tag)).Value;
        }

        protected int? GetSimpleOptionalElementInt(string tag)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (Element.Elements(GetElementName(tag)).Count() > 0)
                return Convert.ToInt32(Element.Element(GetElementName(tag)).Value);
            else
                return null;
        }

        protected string GetSimpleOptionalElementString(string tag)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (Element.Elements(GetElementName(tag)).Count() > 0)
                return Element.Element(GetElementName(tag)).Value;
            else
                return null;
        }

        protected int GetSimpleRequiredElementInt(string tag)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            return Convert.ToInt32(Element.Element(GetElementName(tag)).Value);
        }

        protected bool GetSimpleRequiredElementBool(string tag)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            return Convert.ToBoolean(Element.Element(GetElementName(tag)).Value);
        }

        protected void SetSimpleRequiredElement<T>(string tag, T value, EventHandler eventChanging, EventHandler eventChanged)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (!value.Equals(Element.Element(GetElementName(tag))))
            {
                if (eventChanging != null) eventChanging(this, EventArgs.Empty);

                Element.SetElementValue(GetElementName(tag), value);

                if (eventChanged != null) eventChanged(this, EventArgs.Empty);
            }
        }

        protected void SetSimpleOptionalElement<T>(string tag, T value, EventHandler eventChanging, EventHandler eventChanged)
        {
            if (Element == null) throw new InvalidOperationException("Ancestor element member is null.");
            if (string.IsNullOrEmpty(tag)) throw new ArgumentException("Parameter cannot be null or empty.", "tag");

            if (value == null && Element.Element(GetElementName(tag)) == null)
                return;
            else if (value == null && Element.Element(GetElementName(tag)) != null)
            {
                Element.Element(GetElementName(tag)).Remove();
            }
            else if (value != null && Element.Element(GetElementName(tag)) == null)
            {
                Element.Add(new XElement(GetElementName(tag), value));
            }
            else if (!value.Equals(Element.Element(GetElementName(tag))))
            {
                if (eventChanging != null) eventChanging(this, EventArgs.Empty);

                Element.SetElementValue(GetElementName(tag), value);

                if (eventChanged != null) eventChanged(this, EventArgs.Empty);
            }
        }
    }
}