﻿using System;
using System.Runtime.Serialization;

namespace Sycamores.MobileShare.Communications
{/// <summary>Exposes instance methods for creating, moving, and enumerating through directories and subdirectories. 
    /// This class cannot be inherited.</summary>
    [DataContract(Namespace = "http://janglin.com/Sycamores/MobileShare/Communications")]
    public sealed class DirectoryInfo : FileSystemInfo
    {
        public DirectoryInfo(System.IO.DirectoryInfo info) : base(info) { }

        private new System.IO.DirectoryInfo Info
        {
            get { return (System.IO.DirectoryInfo)base.Info; }
        }

        /// <summary>Gets a value indicating whether the directory exists.</summary>
        [DataMember]
        public override bool Exists
        {
            get { return Info.Exists; }
            internal set { throw new NotSupportedException (); }
        }

        /// <summary>Gets the name of this System.IO.DirectoryInfo instance.</summary>
        [DataMember]
        public override string Name
        {
            get { return Info.Name; }
            internal set { throw new NotSupportedException(); }
        }

        /// <summary>Gets the parent directory of a specified subdirectory.</summary>
        /// <remarks>The parent directory, or null if the path is null or if the file path denotes a root (such as "\", "C:", 
        /// or * "\\server\share").</remarks>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [DataMember]
        public string Parent
        {
            get
            {
                if (Info.Parent == null)
                    return null;
                else
                    return Info.Parent.FullName;
            }
            internal set { throw new NotSupportedException(); }
        }

        /// <summary>Gets the root portion of a path.</summary>
        /// <remarks>A System.IO.DirectoryInfo object representing the root of a path.</remarks>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [DataMember]
        public string Root
        {
            get { return Info.Root.FullName; }
            internal set { throw new NotSupportedException(); }
        }
    }
}