﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.DirectoryServices;
using System.Globalization;
using System.Security.Principal;

using Sycamores.PlatformInvoke;
using Sycamores.PlatformInvoke.Networking;
using Sycamores.PlatformInvoke.Security;

namespace Sycamores.MobileShare.Communications
{
    // NOTE: If you change the class name "Explorer" here, you must also update the reference to "Explorer" in Web.config and in the associated .svc file.
    public class Explorer : IExplorer
    {
        private object Locker = new object();
        private WindowsIdentity _Identity;
        private WindowsImpersonationContext _ImpersonatedUser;
        private UserToken _Token;

        ~Explorer()
        {
            if (_ImpersonatedUser != null)
            {
                _ImpersonatedUser.Undo();

                _ImpersonatedUser.Dispose();
                _Identity.Dispose();
                _Token.Dispose();
            }
        }

        #region IExplorer Members

        public string ThisIsATest(string value)
        {
            return string.Format(CultureInfo.CurrentCulture, "This is only a test: {0}", value);
        }

        public NetworkShareInformation[] GetShares(string computername)
        {

            ShareInformation1[] shares = SystemServicesAccess.GetShares(computername);
            NetworkShareInformation[] result = new NetworkShareInformation[shares.Length];

            int index = 0;

            foreach (ShareInformation1 share in shares)
                result[index++] = new NetworkShareInformation(share);

            return result;
        }

        public string[] GetComputers(string domain)
        {
            ServerInfo[] servers = ManagementAccess.GetServers(ServerType.All, domain);

            IList<string> list = new List<string>();

            foreach (var server in servers)
                list.Add(server.Name);

            return list.ToArray<string>();
        }

        public DirectoryInfo GetDirectoryInfo(string directorypath)
        {
            return new DirectoryInfo(new System.IO.DirectoryInfo(directorypath));
        }

        public string[] GetDomains()
        {
            ServerInfo[] servers = ManagementAccess.GetServers(ServerType.DomainEnum, null);

            IList<string> list = new List<string>();

            foreach (var server in servers)
                list.Add(server.Name);

            return list.ToArray<string>();
        }

        public FileInfo GetFileInfo(string filepath)
        {
            return new FileInfo(new System.IO.FileInfo(filepath));
        }

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public DirectoryInfo[] GetDirectories(string foldername)
        {
            System.IO.DirectoryInfo dirinfo = new System.IO.DirectoryInfo(foldername);
            System.IO.DirectoryInfo[] dirinfos = dirinfo.GetDirectories();
            DirectoryInfo[] output = new DirectoryInfo[dirinfos.Length];

            int count = 0;

            foreach (System.IO.DirectoryInfo systeminfo in dirinfos)
                output[count++] = new DirectoryInfo(systeminfo);

            return output;
        }

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public DirectoryInfo[] GetDirectories(string foldername, string searchPattern)
        {
            if (searchPattern == null)
                return GetDirectories(foldername);
            else
            {
                System.IO.DirectoryInfo dirinfo = new System.IO.DirectoryInfo(foldername);

                System.IO.DirectoryInfo[] dirinfos = dirinfo.GetDirectories(searchPattern);

                DirectoryInfo[] output = new DirectoryInfo[dirinfos.Length];

                int count = 0;

                foreach (System.IO.DirectoryInfo systeminfo in dirinfos)
                    output[count++] = new DirectoryInfo(systeminfo);

                return output;
            }
        }

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <param name="searchOption">One of the values of the System.IO.SearchOption enumeration that specifies whether the search operation
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public DirectoryInfo[] GetDirectories(string foldername, string searchPattern, int? searchOption)
        {
            if (searchPattern == null && !searchOption.HasValue)
                return GetDirectories(foldername);
            else if (searchPattern != null && !searchOption.HasValue)
                return GetDirectories(foldername, searchPattern);
            else if (searchPattern == null && searchOption.HasValue)
                throw new InvalidOperationException("Search Patter cannot be null while there is a search option.");
            else
            {
                System.IO.DirectoryInfo dirinfo = new System.IO.DirectoryInfo(foldername);

                System.IO.DirectoryInfo[] dirinfos = dirinfo.GetDirectories(searchPattern, (System.IO.SearchOption)(searchOption.Value));

                DirectoryInfo[] output = new DirectoryInfo[dirinfos.Length];

                int count = 0;

                foreach (System.IO.DirectoryInfo systeminfo in dirinfos)
                    output[count++] = new DirectoryInfo(systeminfo);

                return output;
            }
        }

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public FileInfo[] GetFiles(string foldername)
        {
            System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

            System.IO.FileInfo[] systeminfos = info.GetFiles();

            FileInfo[] output = new FileInfo[systeminfos.Length];
            int count = 0;

            foreach (System.IO.FileInfo systeminfo in systeminfos)
                output[count++] = new FileInfo(systeminfo);

            return output;
        }

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public FileInfo[] GetFiles(string foldername, string searchPattern)
        {
            if (searchPattern == null)
                return GetFiles(foldername);
            else
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

                System.IO.FileInfo[] systeminfos = info.GetFiles(searchPattern);

                FileInfo[] output = new FileInfo[systeminfos.Length];
                int count = 0;

                foreach (System.IO.FileInfo systeminfo in systeminfos)
                    output[count++] = new FileInfo(systeminfo);

                return output;
            }
        }

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <param name="searchOption">One of the values of the System.IO.SearchOption enumeration that specifies whether the search operation
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public FileInfo[] GetFiles(string foldername, string searchPattern, int? searchOption)
        {
            if (searchPattern == null && !searchOption.HasValue)
                return GetFiles(foldername);
            else if (searchPattern != null && !searchOption.HasValue)
                return GetFiles(foldername, searchPattern);
            else if (searchPattern == null && searchOption.HasValue)
                throw new InvalidOperationException("Search Patter cannot be null while there is a search option.");
            else
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

                System.IO.FileInfo[] systeminfos = info.GetFiles(searchPattern,(System.IO.SearchOption)(searchOption.Value));

                FileInfo[] output = new FileInfo[systeminfos.Length];
                int count = 0;

                foreach (System.IO.FileInfo systeminfo in systeminfos)
                    output[count++] = new FileInfo(systeminfo);

                return output;
            }
        }

        /// <summary>Retrieves an array of strongly typed System.IO.FileSystemInfo objects representing the files and subdirectories matching the specified search criteria.</summary>
        /// <param name="foldername">Folder in which to look.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <returns>An array of strongly typed FileSystemInfo objects matching the search criteria.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The specified path is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public FileSystemInfo[] GetFileSystemInfos(string foldername)
        {
            throw new NotImplementedException();
        }

        /// <summary>Retrieves an array of strongly typed System.IO.FileSystemInfo objects representing the files and subdirectories.</summary>
        /// <param name="foldername">Folder in which to look.</param>
        /// <returns>An array of strongly typed FileSystemInfo objects.</returns>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        public FileSystemInfo[] GetFileSystemInfos(string foldername, string searchPattern)
        {
            throw new NotImplementedException();
        }

        public string ExtractAssociatedIcon(string filepath)
        {
            throw new NotImplementedException();
        }

        /// <summary>Creates a directory.</summary>
        /// <param name="foldername">Directory which to create.</param>
        /// <exception cref="System.IO.IOException">The directory cannot be created.</exception>
        public DirectoryInfo Create(string foldername)
        {
            return Create(foldername, null);
        }

        public DirectoryInfo Create(string folderpath, int? attributes)
        {
            lock (Locker)
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(folderpath);

                info.Create();
                if (attributes.HasValue) info.Attributes = ((System.IO.FileAttributes)attributes.Value);

                return new DirectoryInfo(info);
            }
        }

        public DirectoryInfo CreateSubdirectory(string folderpath, string path)
        {
            return CreateSubdirectory(folderpath, path, null);
        }

        public DirectoryInfo CreateSubdirectory(string folderpath, string path, int? attributes)
        {
            lock (Locker)
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(folderpath);
                System.IO.DirectoryInfo newinfo = info.CreateSubdirectory(path);

                if (attributes.HasValue) newinfo.Attributes = ((System.IO.FileAttributes)attributes.Value);

                return new DirectoryInfo(newinfo);
            }
        }

        public  void Delete(string foldername)
        {
            System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

            info.Delete();
        }

        public void Delete(string foldername, bool? recursive)
        {
            if (recursive.HasValue)
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

                info.Delete(recursive.Value);
            }
            else
                Delete(foldername);
        }

        public void MoveTo(string foldername, string destDirName)
        {
            lock (Locker)
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

                info.MoveTo(destDirName);
            }
        }

        void IExplorer.Create(string foldername, int? attributes)
        {
            lock (Locker)
            {
                System.IO.DirectoryInfo info = new System.IO.DirectoryInfo(foldername);

                info.Create();
                if (attributes.HasValue) info.Attributes = (System.IO.FileAttributes)attributes.Value;
            }
        }

        public string LogOn(string username, string domain, string password)
        {
            Sycamores.PlatformInvoke.Security.UserToken token = new Sycamores.PlatformInvoke.Security.UserToken();

            token.LogonUser(username, domain, password, LogonTypeCodes.Interactive, LogonProviderCodes.Default);

            WindowsIdentity id = new WindowsIdentity(token);

            id.Impersonate();

            return null;
        }

        #endregion
    }
}