﻿using System;
using System.Runtime.Serialization;

namespace Sycamores.MobileShare.Communications
{
    /// <summary>Provides instance methods for the creation, copying, deletion, moving, and opening of files, and aids in the
    /// creation of System.IO.FileStream objects.</summary>
    /// <remarks>This class cannot be inherited.</remarks>
    [DataContract(Namespace = "http://janglin.com/Sycamores/MobileShare/Communications")]
    public sealed class FileInfo : FileSystemInfo
    {
        /// <summary>Default constructor.</summary>
        /// <param name="info">Contained file info or a <see cref="System.IO.FileInfo"/> type./></param>
        public FileInfo(System.IO.FileInfo info) : base(info) { }

        private new System.IO.FileInfo Info
        {
            get { return (System.IO.FileInfo)base.Info; }
        }

        /// <summary> Gets an instance of the parent directory.</summary>
        /// <remarks>A System.IO.DirectoryInfo object representing the parent directory of this file.</remarks>
        /// <exception cref="System.IO.DirectoryNotFoundException">The specified path is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [DataMember]
        public string Directory
        {
            get { return Info.Directory.FullName; }
            internal set { throw new NotImplementedException(); }
        }

        /// <summary>Gets a string representing the directory's full path.</summary>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        /// <exception cref="System.ArgumentNullException">null was passed in for the directory name.</exception>
        [DataMember]
        public string DirectoryName
        {
            get { return Info.DirectoryName; }
            internal set { throw new NotImplementedException(); }
        }

        /// <summary>Gets a value indicating whether a file exists.</summary>
        /// <remarks>Returns true if the file exists; false if the file does not exist or if the file is a directory.</remarks>
        [DataMember]
        public override bool Exists
        {
            get { return Info.Exists; }
            internal set { throw new NotImplementedException(); }
        }

        /// <summary>Gets or sets a value that determines if the current file is read only.</summary>
        /// <exception cref="System.IO.FileNotFoundException">The file described by the current System.IO.FileInfo object could not be found.</exception>
        /// <exception cref="System.IO.IOException">An I/O error occurred while opening the file.</exception>
        /// <exception cref="System.UnauthorizedAccessException">The file described by the current System.IO.FileInfo object is read-only.
        /// <para>This operation is not supported on the current platform.</para>
        /// <para>The caller does not have the required permission.</para></exception>
        [DataMember]
        public bool IsReadOnly
        {
            get { return Info.IsReadOnly; }
            set { Info.IsReadOnly = value; }
        }

        /// <summary>Gets the size, in bytes, of the current file.</summary>
        /// <exception cref="System.IO.IOException">System.IO.FileSystemInfo.Refresh() cannot update the state of the file or directory.</exception>
        /// <exception cref="System.IO.FileNotFoundException">The file does not exist.
        /// <para>The Length property is called for a directory.</para></exception>
        [DataMember]
        public long Length
        {
            get { return Info.Length; }
            internal set { throw new NotImplementedException(); }
        }

        /// <summary>Gets the name of the file.</summary>
        [DataMember]
        public override string Name
        {
            get { return Info.Name; }
            internal set { throw new NotImplementedException(); }
        }

    }
}