﻿using System;
using System.Runtime.Serialization;

namespace Sycamores.MobileShare.Communications
{
[DataContract(Namespace = "http://janglin.com/Sycamores/MobileShare/Communications")]
public abstract class FileSystemInfo
{
    private System.IO.FileSystemInfo mInfo;

    protected System.IO.FileSystemInfo Info { get { return mInfo; } }

    protected FileSystemInfo(System.IO.FileSystemInfo info) { mInfo = info; }

    /// <summary>Gets or sets the System.IO.FileAttributes of the current System.IO.FileSystemInfo.</summary>
    /// <remarks>Uses the int built-in type as the client might not always be a .NET assembly.</remarks>
    /// <exception cref="System.IO.FileNotFoundException">The specified file does not exist.</exception>
    /// <exception cref="System.IO.DirectoryNotFoundException">The specified path is invalid, such as being on an unmapped drive.</exception>
    /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
    /// <exception cref="System.ArgumentException">The caller attempts to set an invalid file attribute.</exception>
    /// <exception cref="System.IO.IOException">System.IO.FileSystemInfo.Refresh() cannot initialize the data.</exception>
    [DataMember]
    public int Attributes
    {
        get { return (int)Info.Attributes; }
        set { Info.Attributes = (System.IO.FileAttributes)value; }
    }
    //
    // Summary:
    //     Gets or sets the creation time of the current System.IO.FileSystemInfo object.
    //
    // Returns:
    //     The creation date and time of the current System.IO.FileSystemInfo object.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.IO.DirectoryNotFoundException:
    //     The specified path is invalid, such as being on an unmapped drive.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string CreationTime
    {
        get { return Info.CreationTime.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.CreationTime = newdate;
        }
    }
    //
    // Summary:
    //     Gets or sets the creation time, in coordinated universal time (UTC), of the
    //     current System.IO.FileSystemInfo object.
    //
    // Returns:
    //     The creation date and time in UTC format of the current System.IO.FileSystemInfo
    //     object.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.IO.DirectoryNotFoundException:
    //     The specified path is invalid, such as being on an unmapped drive.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string CreationTimeUtc
    {
        get { return Info.CreationTimeUtc.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.CreationTimeUtc = newdate;
        }
    }
    //
    // Summary:
    //     Gets a value indicating whether the file or directory exists.
    //
    // Returns:
    //     true if the file or directory exists; otherwise, false.
    [DataMember]
    public abstract bool Exists
    {
        get;
        internal set;
    }
    //
    // Summary:
    //     Gets the string representing the extension part of the file.
    //
    // Returns:
    //     A string containing the System.IO.FileSystemInfo extension.
    [DataMember]
    public string Extension
    {
        get { return Info.Extension; }
        internal set { throw new NotImplementedException(); }
    }
    //
    // Summary:
    //     Gets the full path of the directory or file.
    //
    // Returns:
    //     A string containing the full path.
    //
    // Exceptions:
    //   System.Security.SecurityException:
    //     The caller does not have the required permission.
    [DataMember]
    public virtual string FullName
    {
        get { return Info.FullName; }
        internal set { throw new NotImplementedException(); }
    }
    //
    // Summary:
    //     Gets or sets the time the current file or directory was last accessed.
    //
    // Returns:
    //     The time that the current file or directory was last accessed.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string LastAccessTime
    {
        get { return Info.LastAccessTime.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.LastAccessTime = newdate;
        }
    }
    //
    // Summary:
    //     Gets or sets the time, in coordinated universal time (UTC), that the current
    //     file or directory was last accessed.
    //
    // Returns:
    //     The UTC time that the current file or directory was last accessed.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string LastAccessTimeUtc
    {
        get { return Info.LastAccessTimeUtc.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.LastAccessTimeUtc = newdate;
        }
    }
    //
    // Summary:
    //     Gets or sets the time when the current file or directory was last written
    //     to.
    //
    // Returns:
    //     The time the current file was last written.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string LastWriteTime
    {
        get { return Info.LastWriteTime.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.LastWriteTime = newdate;
        }
    }
    //
    // Summary:
    //     Gets or sets the time, in coordinated universal time (UTC), when the current
    //     file or directory was last written to.
    //
    // Returns:
    //     The UTC time when the current file was last written to.
    //
    // Exceptions:
    //   System.IO.IOException:
    //     System.IO.FileSystemInfo.Refresh() cannot initialize the data.
    //
    //   System.PlatformNotSupportedException:
    //     The current operating system is not Microsoft Windows NT or later.
    [DataMember]
    public string LastWriteTimeUtc
    {
        get { return Info.LastWriteTimeUtc.ToString(); }
        set
        {
            DateTime newdate;
            DateTime.TryParse(value, out newdate);
            Info.LastWriteTimeUtc = newdate;
        }
    }
    //
    // Summary:
    //     For files, gets the name of the file. For directories, gets the name of the
    //     last directory in the hierarchy if a hierarchy exists. Otherwise, the Name
    //     property gets the name of the directory.
    //
    // Returns:
    //     A string that is the name of the parent directory, the name of the last directory
    //     in the hierarchy, or the name of a file, including the file name extension.
    [DataMember]
    public abstract string Name
    {
        get;
        internal set;
    }
}
}