﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

namespace Sycamores.MobileShare.Communications
{
    // NOTE: If you change the interface name "IEntrance" here, you must also update the reference to "IEntrance" in Web.config.
    [ServiceContract(Namespace = "http://janglin.com/Sycamores/MobileShare/Communications")]
    public interface IEntrance
    {
        [OperationContract]
        string ThisIsATest(string value);

        [OperationContract]
        string[] GetDomains();

        [OperationContract]
        string[] GetComputers(string domain);

        [OperationContract]
        string[] GetShares(string computername);

        [OperationContract]
        DirectoryInfo GetDirectoryInfo(string directorypath);

        [OperationContract]
        FileInfo GetFileInfo(string filepath);

        [OperationContract]
        string ExtractAssociatedIcon(string filepath);

        [OperationContract]
        string LogOn(string username, string domain, string password);

        #region DirectoryInfo

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        DirectoryInfo[] GetDirectories(string foldername);

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        DirectoryInfo[] GetDirectories(string foldername, string searchPattern);

        /// <summary>Returns an array of directories in the current System.IO.DirectoryInfo matching the given search criteria and using
        /// a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <param name="searchOption">One of the values of the System.IO.SearchOption enumeration that specifies whether the search operation
        /// should include only the current directory or should include all subdirectories</param>
        /// <returns>An array of type DirectoryInfo matching searchPattern.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [OperationContract]
        DirectoryInfo[] GetDirectories(string foldername, string searchPattern, int? searchOption);

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        FileInfo[] GetFiles(string foldername);

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        FileInfo[] GetFiles(string foldername, string searchPattern);

        /// <summary>Returns a file list from the current directory matching the given searchPattern and using a value to determine whether to search subdirectories.</summary>
        /// <param name="foldername">Directory in which to search.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <param name="searchOption">One of the values of the System.IO.SearchOption enumeration that specifies whether the search operation
        /// <returns>An array of type System.IO.FileInfo.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The path encapsulated in the DirectoryInfo object is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [OperationContract]
        FileInfo[] GetFiles(string foldername, string searchPattern, int? searchOption);

        /// <summary>Retrieves an array of strongly typed System.IO.FileSystemInfo objects representing the files and subdirectories matching the specified search criteria.</summary>
        /// <param name="foldername">Folder in which to look.</param>
        /// <param name="searchPattern">The search string, such as "System*", used to search for all directories beginning with the word "System".</param>
        /// <returns>An array of strongly typed FileSystemInfo objects matching the search criteria.</returns>
        /// <exception cref="System.IO.DirectoryNotFoundException">The specified path is invalid, such as being on an unmapped drive.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        FileSystemInfo[] GetFileSystemInfos(string foldername);

        /// <summary>Retrieves an array of strongly typed System.IO.FileSystemInfo objects representing the files and subdirectories.</summary>
        /// <param name="foldername">Folder in which to look.</param>
        /// <returns>An array of strongly typed FileSystemInfo objects.</returns>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        [OperationContract]
        FileSystemInfo[] GetFileSystemInfos(string foldername, string searchPattern);

        /// <summary>Creates a directory.</summary>
        /// <param name="foldername">Path of directory to be created.</param>
        /// <exception cref="System.Security.IOException">The directory cannot be created.</exception>
        [OperationContract]
        void Create(string foldername, int? attributes);

        /// <summary>Creates a subdirectory or subdirectories on the specified path. The specified path can be relative to this instance of the System.IO.DirectoryInfo class.</summary>
        /// <param name="path">The specified path. This cannot be a different disk volume or Universal Naming Convention (UNC) name.</param>
        /// <remarks>Returns the last directory specified in path.</remarks>
        /// <param name="foldername">Path of directory to be created.</param>
        /// <exception cref="System.ArgumentException">path does not specify a valid file path or contains invalid DirectoryInfo characters.</exception>
        /// <exception cref="System.ArgumentNullException">path is null.</exception>
        /// <exception cref="System.IO.PathTooLongException">The specified path, file name, or both exceed the system-defined maximum
        /// length. For example, on Windows-based platforms, paths must be less than 248 characters, and file names must be less than
        /// 260 characters. The specified path, file name, or both are too long.</exception>
        /// <exception cref="System.Security.IOException"> The subdirectory cannot be created.
        /// A file or directory already has the name specified by path.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have code access permission to create the directory.
        /// The caller does not have code access permission to read the directory described by the returned System.IO.DirectoryInfo object. 
        /// This can occur when the pathparameter describes an existing directory.</exception>
        /// <exception cref="System.NotSupportedException">path contains a colon character (:) that is not part of a drive label ("C:\").</exception>
        [OperationContract]
        DirectoryInfo CreateSubdirectory(string foldername, string path, int? attributes);

        //
        // Summary:
        //     Deletes this System.IO.DirectoryInfo if it is empty.
        //
        // Exceptions:
        //   System.IO.IOException:
        //     The directory is not empty. -or-The directory is the application's current
        //     working directory.
        //
        //   System.Security.SecurityException:
        //     The caller does not have the required permission.
        //
        //   System.IO.DirectoryNotFoundException:
        //     The directory described by this System.IO.DirectoryInfo object does not exist
        //     or could not be found.
        void Delete(string foldername);

        //
        // Summary:
        //     Deletes this instance of a System.IO.DirectoryInfo, specifying whether to
        //     delete subdirectories and files.
        //
        // Parameters:
        //   recursive:
        //     true to delete this directory, its subdirectories, and all files; otherwise,
        //     false.
        //
        // Exceptions:
        //   System.IO.IOException:
        //     The directory is read-only.-or- The directory contains one or more files
        //     or subdirectories and recursive is false.-or-The directory is the application's
        //     current working directory.
        //
        //   System.Security.SecurityException:
        //     The caller does not have the required permission.
        [OperationContract]
        void Delete(string foldername, bool? recursive);

        /// <summary>Moves a System.IO.DirectoryInfo instance and its contents to a new path.</summary>
        /// <param name="foldername">Source path.</param>
        /// <param name="destDirName">The name and path to which to move this directory. The destination cannot
        /// be another disk volume or a directory with the identical name. It can be
        /// an existing directory to which you want to add this directory as a subdirectory.</param>
        /// <exception cref="System.ArgumentNullException">destDirName is null.</exception>
        /// <exception cref="System.ArgumentException">destDirName is an empty string (''").</exception>
        /// <exception cref="System.IO.IOException">An attempt was made to move a directory to a different volume.
        /// or destDirName already exists.
        /// or You are not authorized to access this path.
        /// or The directory being moved and the destination directory have the same name.</exception>
        /// <exception cref="System.Security.SecurityException">The caller does not have the required permission.</exception>
        /// <exception cref="System.IO.DirectoryNotFoundException">The destination directory cannot be found.</exception>
        [OperationContract]
        void MoveTo(string foldername, string destDirName);

        #endregion
    }
}