﻿using System;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml.Linq;
using System.IO;
using System.ServiceProcess;
using System.Globalization;

using Sycamores.MobileShare.Communications;

namespace Sycamores.MobileShare.Service.Configuration
{
    public partial class MainForm : Form
    {
        XDocument mAppconfig;
        XAttribute mBaseaddress;

        /// <summary>Default constructor.</summary>
        public MainForm() { InitializeComponent(); }

        private void MainForm_Load(object sender, EventArgs e)
        {

#if DEBUG
            if (!File.Exists("MobileShareServiceHost.exe.config"))
                File.Copy(@"D:\Users\danp\Documents\Visual Studio 2010\Projects\MobileShare\ServiceApplication\Service\bin\Debug\MobileShareServiceHost.exe.config", "MobileShareServiceHost.exe.config");
#endif

            mAppconfig = XDocument.Load("MobileShareServiceHost.exe.config");

            XElement root = mAppconfig.Root;

            XElement serviceModel = root.Element("system.serviceModel");
            XElement services = serviceModel.Elements("services").FirstOrDefault();

            var query = from service in services.Elements("service")
                        where service.Attribute("name").Value.Equals("Sycamores.MobileShare.Communications.Entrance")
                        select service;

            XElement serviceE = query.First();
            XElement host = serviceE.Element("host");
            XElement baseaddresses = host.Element("baseAddresses");
            mBaseaddress = baseaddresses.Element("add").Attribute("baseAddress");

            Uri addressuri = new Uri(mBaseaddress.Value);

            switch (addressuri.Scheme)
            {
                case "http":
                    TypeComboBox.SelectedIndex = 0;
                    break;
            }

            AddressTextBox.Text = addressuri.Host;
            LocalTextBox.Text = addressuri.AbsolutePath;
            PortTextBox.Text = addressuri.Port.ToString(CultureInfo.InvariantCulture);
        }

        private void CloselButton_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;

            try
            {
                switch (MessageBox.Show(this
                    , Properties.Resources.LabelChanges
                    , Properties.Resources.DialogTitle
                    , MessageBoxButtons.YesNo
                    , MessageBoxIcon.Question
                    , MessageBoxDefaultButton.Button2
                    , MessageBoxOptions.DefaultDesktopOnly))
                {
                    case DialogResult.Yes:
                        this.Close();
                        break;
                    case DialogResult.No:
                        break;
                }
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        //private void TestButton_Click(object sender, EventArgs e)
        //{
        //    Cursor.Current = Cursors.WaitCursor;

        //    try
        //    {
        //        Uri[] uris = new Uri[1];
        //        uris[0] = GetUri();
        //        ServiceHost mServiceHost = new ServiceHost(typeof(Sycamores.MobileShare.Communications.Entrance),uris);

        //        mServiceHost.AddServiceEndpoint(typeof(IEntrance), new BasicHttpBinding(), GetUri().ToString());

        //        mServiceHost.Open();

        //        mServiceHost.Close();

        //        mServiceHost = null;
        //    }
        //    catch (Exception ex)
        //    {
        //        MessageBox.Show(Properties.Resources.LabelTestFail
        //       , Properties.Resources.DialogTitle
        //       , MessageBoxButtons.OK
        //       , MessageBoxIcon.Error);

        //        return;
        //    }
        //    finally
        //    {
        //        Cursor.Current = Cursors.Default;
        //    }

        //    MessageBox.Show(Properties.Resources.LabelTestSuccess
        //        , Properties.Resources.DialogTitle
        //        , MessageBoxButtons.OK
        //        , MessageBoxIcon.Information);
        //}

        private Uri GetUri()
        {
            CultureInfo cultureinvariant = CultureInfo.InvariantCulture;
            System.Text.StringBuilder newuri = new StringBuilder();

            newuri.Append("http://");
            newuri.Append(AddressTextBox.Text);
            newuri.Append(":");
            newuri.Append(Convert.ToInt32(PortTextBox.Text, cultureinvariant).ToString(cultureinvariant));
            newuri.Append(LocalTextBox.Text);

            Uri output = new Uri(newuri.ToString());

            return output;
        }

        private void OkButton_Click(object sender, EventArgs e)
        {
            CultureInfo cultureinvariant = CultureInfo.InvariantCulture;
            Cursor.Current = Cursors.WaitCursor;

            ServiceController service = new ServiceController("Mobile Share Service Host");

            TimeSpan timeout = TimeSpan.FromMilliseconds(30000);

            try
            {
                if (service.Status.Equals(ServiceControllerStatus.Running)) service.Stop();

                service.WaitForStatus(ServiceControllerStatus.Stopped, timeout);

                System.Text.StringBuilder newuri = new StringBuilder();

                newuri.Append("http://");
                newuri.Append(AddressTextBox.Text);
                newuri.Append(":");
                newuri.Append(Convert.ToInt32(PortTextBox.Text, cultureinvariant).ToString(cultureinvariant));
                newuri.Append(LocalTextBox.Text);

                mBaseaddress.SetValue(newuri.ToString());
                mAppconfig.Save("MobileShareServiceHost.exe.config");

                service.Start();

                service.WaitForStatus(ServiceControllerStatus.Running, timeout);

            }
            finally
            {
                service.Dispose();
                Cursor.Current = Cursors.Default;
            }

            MessageBox.Show(this
                , Properties.Resources.LabelSuccess
                , Properties.Resources.DialogTitle
                , MessageBoxButtons.OK
                , MessageBoxIcon.Information
                , MessageBoxDefaultButton.Button1
                , MessageBoxOptions.DefaultDesktopOnly);
            this.Close();
        }
    }
}