﻿using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;

using Sycamores.PlatformInvoke.SystemServices;
using Nws = Sycamores.PlatformInvoke.SystemServices.NetworkShareManagement;

[assembly: CLSCompliant(true)]
namespace Sycamores.PlatformInvoke
{
    public partial class SystemServicesAccess
    {
        public static ShareInformation1[] GetShares(string server)
        {
            IntPtr shareinfo = IntPtr.Zero;
            int entriesread;
            int count;
            int result;
            int resume = 0;

            try
            {
                result = Nws.NetShareEnum(server
                    , NetworkShareManagement.ShareLevelCodes.One
                    , out shareinfo
                    , Nws.MAX_PREFERRED_LENGTH
                    , out entriesread
                    , out count
                    , ref resume);

                switch (result)
                {
                    case 0:
                        Type sharetype = typeof(Nws.SHARE_INFO_1);
                        int offset = Marshal.SizeOf(sharetype);
                        ShareInformation1[] shares = new ShareInformation1[entriesread];

                        for (int i = 0, item = shareinfo.ToInt32(); i < entriesread; i++, item += offset)
                        {
                            IntPtr pitem = new IntPtr(item);
                            Nws.SHARE_INFO_1 si = (Nws.SHARE_INFO_1)Marshal.PtrToStructure(pitem, sharetype);
                            shares[i].NetName  = si.NetName;
                            shares[i].ShareType = si.ShareType;
                            shares[i].Remark = si.Remark;
                        }

                        return shares;
                    default:
                        throw new InvalidOperationException();
                }
            }
            catch { throw; }
            finally
            {
                if (!shareinfo.Equals(IntPtr.Zero))
                {
                    Sycamores.PlatformInvoke.Networking.Management.NetApiBufferFree(shareinfo);
                    shareinfo = IntPtr.Zero;
                }
            }
        }
    }

    public struct ShareInformation1
    {
        public string NetName;
        public ShareTypeCode ShareType;
        public string Remark;
    }

    /// <summary>A bitmask of flags that specify the type of share.</summary>
    /// <remarks>Calls to the NetShareSetInfo function ignore this member.</remarks>
    [Flags]
    public enum ShareTypeCode : uint
    {
        /// <summary>Disk Drive.</summary>
        DiskTree = 0x00000000,
        /// <summary>Print Queue.</summary>
        PrintQueue = 0x00000001,
        /// <summary>Communication device.</summary>
        Device = 0x00000002,
        /// <summary>Interprocess communication (IPC).</summary>
        Ipc = 0x00000003,
        /// <summary>Special share reserved for interprocess communication (IPC$) or remote administration of the server (ADMIN$). 
        /// Can also refer to administrative shares such as C$, D$, E$, and so forth. For more information, 
        /// see the network share functions.</summary>
        Special = 0x80000000,
        /// <summary>A temporary share.</summary>
        Temporary = 0x40000000,
    }
}