﻿using System;
using System.Runtime.InteropServices;

namespace Sycamores.PlatformInvoke.SystemServices
{
    internal class NetworkShareManagement
    {
        internal const int MAX_PREFERRED_LENGTH = -1;

        /// <summary>Share information, NT, level 0</summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        internal struct SHARE_INFO_0
        {
            [MarshalAs(UnmanagedType.LPWStr)]
            public string NetName;
        }

        /// <summary>Share information, NT, level 1</summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        internal struct SHARE_INFO_1
        {
            [MarshalAs(UnmanagedType.LPWStr)]
            public string NetName;
            public ShareTypeCode ShareType;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string Remark;
        }

        /// <summary>Share information, NT, level 2</summary>
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        internal struct SHARE_INFO_2
        {
            [MarshalAs(UnmanagedType.LPWStr)]
            public string NetName;
            public ShareTypeCode ShareType;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string Remark;
            public int Permissions;
            public int MaxUsers;
            public int CurrentUsers;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string Path;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string Password;
        }

        [StructLayoutAttribute(LayoutKind.Sequential)]
        internal struct SECURITY_DESCRIPTOR
        {
            public byte revision;
            public byte size;
            public short control;
            public IntPtr owner;
            public IntPtr group;
            public IntPtr sacl;
            public IntPtr dacl;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        struct SHARE_INFO_502
        {
            public string shi502_netname;
            [MarshalAs(UnmanagedType.U4)]
            public ShareTypeCode shi502_type;
            public string shi502_remark;
            /// <summary>used w/ share level security only</summary>
            [MarshalAs(UnmanagedType.U4)]
            public int shi502_permissions;
            [MarshalAs(UnmanagedType.U4)]
            public int shi502_max_uses;
            [MarshalAs(UnmanagedType.U4)]
            public int shi502_current_uses;
            public string shi502_path;
            /// <summary>used w/ share level security only</summary>
            public string shi502_passwd;
            [MarshalAs(UnmanagedType.U4)]
            public int shi502_reserved;
            public IntPtr shi502_security_descriptor;
        }

        /// <summary>Enumerate shares (NT)</summary>
        [DllImport("netapi32", CharSet = CharSet.Unicode)]
        internal static extern int NetShareEnum(string lpServerName
            , ShareLevelCodes dwLevel
            , out IntPtr lpBuffer
            , int dwPrefMaxLen
            , out int entriesRead
            , out int totalEntries
            , ref int hResume);

        /// <summary>Enumerate shares (9x)</summary>
        [DllImport("svrapi", CharSet = CharSet.Ansi)]
        internal static extern int NetShareEnum([MarshalAs(UnmanagedType.LPTStr)] string lpServerName
            , ShareLevelCodes dwLevel
            , IntPtr lpBuffer
            , ushort cbBuffer
            , out ushort entriesRead
            , out ushort totalEntries);

        internal enum ShareLevelCodes : int
        {
            /// <summary>Return share names.</summary>
            /// <remarks>The bufptr parameter points to an array of SHARE_INFO_0 structures.</remarks>
            Zero = 0,
            /// <summary>Return information about shared resources, including the name and type of the resource, 
            /// and a comment associated with the resource.</summary>
            /// <remarks>The bufptr parameter points to an array of SHARE_INFO_1 structures.</remarks>
            One = 1,
            /// <summary>Return information about shared resources, including name of the resource, type and permissions, password, 
            /// and number of connections.</summary>
            /// <remarks>The bufptr parameter points to an array of SHARE_INFO_2 structures.</remarks>
            Two = 2,
            /// <summary>Return information about shared resources, including name of the resource, type and permissions, number of connections, 
            /// and other pertinent information.</summary>
            /// <remarks>The bufptr parameter points to an array of SHARE_INFO_502 structures.
            /// <para>Shares from different scopes are not returned. 
            /// For more information about scoping, see the Remarks section of the documentation for the NetServerTransportAddEx function.</para></remarks>
            FiveOTwo = 502,
            /// <summary>Return information about shared resources, including the name of the resource, type and permissions, 
            /// number of connections, and other pertinent information. 
            /// </summary>
            /// <remarks>The bufptr parameter points to an array of SHARE_INFO_503 structures. 
            /// <para>Shares from all scopes are returned. If the shi503_servername member of this structure is "*", 
            /// there is no configured server name and the NetShareEnum function enumerates shares for all the unscoped names.</para>
            /// <para>Windows Server 2003, Windows XP, Windows 2000 Server, and Windows 2000 Professional:  This information level is not supported.</para></remarks>
            FiveOThree = 503,
        }
    }
}