﻿using System;
using System.Runtime.InteropServices;
using System.Drawing;

namespace Sycamores.PlatformInvoke
{
    public class WindowsUserInterface
    {
        [DllImport("shell32.dll", CharSet = CharSet.Auto)]
        internal static extern uint ExtractIconEx(string szFileName
            , int nIconIndex
            , IntPtr[] phiconLarge
            , IntPtr[] phiconSmall
            , uint nIcons);

        [DllImport("user32.dll")]
        internal static extern bool DestroyIcon(IntPtr hIcon);
        
        public static Icon ExtractIconFromExe(string file, int index, bool large)
        {
            unsafe
            {
                uint readIconCount = 0;
                IntPtr[] hDummy = new IntPtr[1] { IntPtr.Zero };
                IntPtr[] hIconEx = new IntPtr[1] { IntPtr.Zero };

                try
                {
                    if (large)
                        readIconCount = ExtractIconEx(file, index, hIconEx, hDummy, 1);
                    else
                        readIconCount = ExtractIconEx(file, index, hDummy, hIconEx, 1);

                    if (readIconCount > 0 && hIconEx[0] != IntPtr.Zero)
                    {
                        // GET FIRST EXTRACTED ICON
                        Icon extractedIcon = (Icon)Icon.FromHandle(hIconEx[0]).Clone();

                        return extractedIcon;
                    }
                    else // NO ICONS READ
                        return null;
                }
                catch (Exception ex)
                {
                    /* EXTRACT ICON ERROR */

                    // BUBBLE UP
                    throw new ApplicationException("Could not extract icon", ex);
                }
                finally
                {
                    // RELEASE RESOURCES
                    foreach (IntPtr ptr in hIconEx)
                        if (ptr != IntPtr.Zero) DestroyIcon(ptr);

                    foreach (IntPtr ptr in hDummy)
                        if (ptr != IntPtr.Zero) DestroyIcon(ptr);
                }
            }
        }
    }
}