﻿using System;
using System.Diagnostics;
using System.ServiceProcess;
using System.ServiceModel;
using System.Threading;

using Sycamores.MobileShare.Communications;
using Res = Sycamores.MobileShare.Service.Properties.Resources;

namespace Sycamores.MobileShare.Service
{
    /// <summary>Service hosting communications service.</summary>
    public partial class Host : ServiceBase
    {
        private ServiceHost mServiceHost;

        /// <summary>Default constructor.</summary>
        public Host() { InitializeComponent(); }

        /// <summary>On start.</summary>
        /// <param name="args">Argument array.</param>
        protected override void OnStart(string[] args)
        {
#if DEBUG
            Thread.Sleep(15000);
#endif
            try
            {
                if (mServiceHost != null) mServiceHost.Close();

                mServiceHost = new ServiceHost(typeof(Explorer));

                mServiceHost.Open();
            }
            catch (Exception ex)
            {
                Log(ex.Message);
            }
        }

        /// <summary>On stop.</summary>
        protected override void OnStop()
        {
            try
            {
                if (mServiceHost != null)
                {
                    mServiceHost.Close();
                    mServiceHost = null;
                }
            }
            catch (Exception ex)
            {
                Log(ex.Message);
            }
        }

        internal static void Log(string message)
        {
            using (System.Diagnostics.EventLog appLog = new System.Diagnostics.EventLog())
            {
                appLog.Source = Res.ApplicationTitle;
                appLog.WriteEntry(message, EventLogEntryType.Error);
            }
        }
    }
}