﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Xml.Linq;

namespace Sycamores.XFramework
{
    public abstract class KeyedComplexTypeCollection<TKey, TValue, TEventArgs>
        : Entity, IDictionary<TKey, TValue>
        where TValue : ComplexType
        where TEventArgs : EventArgs
    {
        private XElement mElement;

        protected XElement Element { get { return mElement; } }

        private IDictionary<TKey, TValue> mDictionary = new Dictionary<TKey, TValue>();

        protected KeyedComplexTypeCollection(string nameSpace, XElement element)
            : base(nameSpace)
        {
            mElement = element;
        }

        public event EventHandler<TEventArgs> ItemRemoving;
        public event EventHandler<TEventArgs> ItemRemoved;

        protected IDictionary<TKey, TValue> Dictionary
        {
            get { return mDictionary; }
        }

        public abstract void Add(XName tag, TValue value);

        #region IDictionary<TKey,TValue> Members

        public void Add(TKey key, TValue value)
        {
            mDictionary.Add(key, value);
        }

        public bool ContainsKey(TKey key)
        {
            return mDictionary.ContainsKey(key);
        }

        public ICollection<TKey> Keys
        {
            get { return mDictionary.Keys; }
        }

        public bool Remove(TKey key)
        {
            return mDictionary.Remove(key);
        }

        public bool TryGetValue(TKey key, out TValue value)
        {
            return mDictionary.TryGetValue(key, out value);
        }

        public ICollection<TValue> Values
        {
            get { return mDictionary.Values; }
        }

        public TValue this[TKey key]
        {
            get
            {
                foreach (var value in mDictionary.Values)
                    System.Diagnostics.Debug.WriteLine(value.ToString());

                return mDictionary[key];
            }
            set
            {
                mDictionary[key] = value;
            }
        }

        #endregion

        #region ICollection<KeyValuePair<TKey,TValue>> Members

        public void Add(KeyValuePair<TKey, TValue> item)
        {
            mDictionary.Add(item);
        }

        public void Clear()
        {
            mDictionary.Clear();
        }

        public bool Contains(KeyValuePair<TKey, TValue> item)
        {
            return mDictionary.Contains(item);
        }

        public void CopyTo(KeyValuePair<TKey, TValue>[] array, int arrayIndex)
        {
            mDictionary.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return mDictionary.Count; }
        }

        public bool IsReadOnly
        {
            get { return mDictionary.IsReadOnly; }
        }

        public bool Remove(KeyValuePair<TKey, TValue> item)
        {
            return mDictionary.Remove(item);
        }

        #endregion

        #region IEnumerable<KeyValuePair<TKey,TValue>> Members

        public IEnumerator<KeyValuePair<TKey, TValue>> GetEnumerator()
        {
            return mDictionary.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return mDictionary.GetEnumerator();
        }

        #endregion
    }
}