﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using System.Xml.Linq;

[assembly: CLSCompliant(true)]
namespace Sycamores.XFramework
{
    /// <summary>Validating XML file.</summary>
    public abstract class ValidatingXmlFile
    {
        private string mFilename;
        private XDocument mDocument;
        private XmlSchemaSet mSchemas;

        /// <summary>Name of the XML file.</summary>
        protected string Filename { get { return mFilename; } }

        /// <summary>XML document contained in the file.</summary>
        protected XDocument Document { get { return mDocument; } }

        protected ValidatingXmlFile(string nameSpace, string rootTag, string filepath, string schemaFilepath)
        {
            mSchemas = new XmlSchemaSet();
            mSchemas.Add(nameSpace, XmlReader.Create(new StringReader(schemaFilepath)));

            mFilename = filepath;

            if (File.Exists(Filename))
            {
                mDocument = XDocument.Load(Filename);
                mDocument.Validate(mSchemas
                    , (sender, e) => { throw new XmlSchemaValidationException(e.Message, e.Exception); }
                    , true);

            }
            else
                mDocument = new XDocument();

            if (mDocument.Root == null) mDocument.Add(new XElement(rootTag));
        }

        public void Save() { mDocument.Save(Filename); }
    }
}